#include "xkra.h"
#include "black_scholes_proto.h"
#include "statistic_proto.h"
#include <math.h>

double option_value_black_scholes(\
float current_stock_price,\
float option_strike_price,\
float riskless_interest,\
int days_to_expiration,\
double expected_volatility,\
float *present_call_value,\
float *present_put_value\
)
{
/*
C = S . N(d1) - K . exp(-rt) . N(d2)
d1 = ( (ln(S) / K . exp(-rt) ) / volatility . sqrt(t) ) +\
 .5 . volatility . sqrt(t)
d2 = d1 - volatility sqrt(t)

for example:
on Nov 1 1990 we want to price the call Dec 1990 Disney stock
(assume no div.).
r = interest rate = 7.12%
t = days to expiration = 50
S = stock price = 92.00$
K = stike price = 95.00$ 
expected_volatility = 35%
*/

double part_of_year;
double present_value_strike_price;
double d1, d2;
double Nd1, Nd2;

if(debug_flag)
	{
	fprintf(stdout,\
	"option_value_black_scholes(): arg\n\
	current_stock_price=%f\n\
	option_strike_price=%f\n\
	riskless_interest=%f\n\
	days_to_expiration=%d\n\
	expected_volatility=%f\n",\
	current_stock_price,\
	option_strike_price,\
	riskless_interest,\
	days_to_expiration,\
	expected_volatility);
	}

/*0) compute part of the year */
part_of_year = days_to_expiration / 365.0;
if(debug_flag)
	{
	fprintf(stdout, "option_value_black_scholes(): part_of_year=%.4f\n",\
	part_of_year);
	}

/*1) compute present value of strike price */
present_value_strike_price =\
option_strike_price * exp (-1.0 * riskless_interest * part_of_year); 
if(debug_flag)
	{
	fprintf(stdout,\
	"option_value_black_scholes(): present_value_strike_price=%.6f\n",\
	(float)present_value_strike_price);
	}

/*2) compute d1 */
d1 =\
(log(current_stock_price / present_value_strike_price) / \
(expected_volatility * sqrt(part_of_year) ) ) +\
.5 * expected_volatility * sqrt(part_of_year);
if(debug_flag)
	{
	fprintf(stdout, "option_value_black_scholes(): d1=%.6f\n", (float)d1);
	}

/*3) compute d2 */
d2 =\
d1 - expected_volatility * sqrt(part_of_year);
if(debug_flag)
	{
	fprintf(stdout, "option_value_black_scholes(): d2=%.6f\n", (float)d2);
	}

/*4) compute Nd1 use table */
/*Nd1 = .4570;*/
Nd1 = get_probability(d1);
if(debug_flag)
	{
	fprintf(stdout, "option_value_black_scholes(): Nd1=%.6f\n", (float)Nd1);
	}

/*5) compute Nd2 use table*/
/*Nd2 = .4061;*/
Nd2 = get_probability(d2);
if(debug_flag)
	{
	fprintf(stdout, "option_value_black_scholes(): Nd2=%.6f\n", (float)Nd2);
	}

/*6) compute C */
*present_call_value =\
current_stock_price * Nd1 - option_strike_price * Nd2;
if(debug_flag)
	{
	fprintf(stdout,\
	"option_value_black_scholes(): present_call_value=%f\n", (float)
	*present_call_value);
	}

/*
Use the Put-Call parity to calculate the value of the put with the same
strike price and the same expiration date as the call.

P = C + K . exp(-rt) - S
*/

/* So P = C + present_value_strike_price - stock_price */

*present_put_value =\
*present_call_value + present_value_strike_price - current_stock_price;
if(debug_flag)
	{
	fprintf(stdout,\
	"option_value_black_scholes(): present_call_value=%f\n",
	*present_put_value);
	}

/* return OK */
return 1;
}/* end function option_value_black_scholes */


