/*
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "xkra.h"

FD_xkra *fdui;


static float data[DATA_SIZE];
static int data_points;
static old_current_stock_price;
static char *old_fundname;
static double old_yearly_standard_deviation;

int get_days_to_expiration(\
char *expiration_month, char *expiration_year, float *days_to_expiration)
{
/*
This calculates the days until expiration given at least the first 3
characters of the name of the month in the English language.
It uses the computer time as the current time.
The year MUST be in the form 1999, 0r 2000, so 4 digits.
The expiration is taken to happen on the third friday of the month.
Change as required.
*/

struct tm *petm;
time_t now, expiration_time;
int exp_month;
int exp_year;
int seconds_per_day;
int mday;
int friday_cnt;

if(debug_flag)
	{
	fprintf(stdout,\
"get_days_to_expiration(): arg expiration_month=%s expiration_year=%s\n",\
	expiration_month, expiration_year);
	}

/* argument check */
if(! expiration_month) return 0;
if(! expiration_year) return 0;

/* get computer time */
now = time(NULL);

/* get expiration time from function arguments */
/* allocate space for a structure tm */
petm = (struct tm *) malloc(sizeof(struct tm) );
if(! petm)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
	"get_days_to_expiration(): cannot allocate space for structure petm\n");
		}
	return 0;
	}

exp_month = month_to_digit(expiration_month);
if(debug_flag)
	{
	fprintf(stdout,\
	"get_days_to_expiration(): exp_month=%d\n", exp_month);
	}
if(exp_month == 0) return 0;

exp_year = atoi(expiration_year);
if(debug_flag)
	{
	fprintf(stdout,\
	"get_days_to_expiration(): exp_year=%d\n", exp_year);
	}

/*
need tm_wday for check on friday (sunday = 0), then iterate forward :)
so friday = 5
*/
friday_cnt = 0;
mday = 1;

while(1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "get_days_to_expiration(): mday=%d\n", mday);
		}

	petm -> tm_sec = 0;/* unknown */
	petm -> tm_min = 0;/* unknown */
	petm -> tm_hour = 12;/* mid day */
	petm -> tm_mday = mday; /* 1 to 31, use 3rd friday in month */
	petm -> tm_mon = exp_month - 1;
	petm -> tm_year = exp_year - 1900;
	petm -> tm_isdst = -1;/* daylight savings time n.a. */
	/* the following are not implemented (see /usr/include/time.h) */
	/* petm -> tm_gmtoff = 0;*/
	/* strcp(petm -> tm_zone, "");/* time zone 3 letters */

	expiration_time = mktime(( struct tm *) petm);
	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration(): expiration_time=%lu\n", expiration_time);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration(): at wday=%d\n", petm -> tm_wday);
		}

	/* check for friday  (day 5 in week) */

	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration(): petm -> tm_wday=%d\n",\
		petm -> tm_wday);
		}

	if(petm -> tm_wday == 5)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"get_days_to_expiration(): iterating friday_cnt=%d\n",\
			friday_cnt);
			}
  		friday_cnt++;
		/* third friday in expiration month is expiration day */
		if(friday_cnt == 3) break;
		}
	mday++;
	if(mday > 31)
		{
		free(petm);
		return 0;
		}
	}/* end while looking for 3rd friday in expiration month */

/* calculate days in year to third friday in expiration month */
seconds_per_day = 60 * 60 * 24;
*days_to_expiration = (float) (expiration_time - now) / seconds_per_day;
if(debug_flag)
	{
	fprintf(stdout,\
	"get_days_to_expiration(): days_to_expiration=%.2f\n",\
	*days_to_expiration);
	}

free(petm);

return 1;
} /* end function get_days_to_expiration */


int load_stock_prices(char *filename, float *data, int *data_points)
{
int a, b, i, j;
FILE *stockfile;
char fiwd[80], fimo[80], si[80], temp[80];
float trend, fmink, fmaxk;
int fida, af, pg, sp, xpos, ypos, lenfie, lenfr, specialfield;
int special_field_type;
float  value, closeprice, openprice, minprice, maxprice, volume;
int header_flag;
char header[80];
int month, day, year;
char field[6][80];
char date[80];

if(debug_flag)
	{
	fprintf(stdout, "load_stock_prices(): arg filename=%s *data=%ld\n",\
	filename, (long)data, (long)data_points);
	}

/* argument check */
if(! filename) return 0;
if(! data) return 0;
if(! data_points) return 0;

*data_points = -1;
stockfile = fopen(filename, "r");
if(! stockfile) 
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"load_stock_prices(): cannot open file %s for read\n", filename);
		}
	return (0);
	}

/* get header from file */
fscanf(stockfile,"%s %s %d %s %f %d %d %d %d %d %d %f %f %d",\
	fiwd,fimo,&fida,si,&trend,&af,&pg,&sp,&xpos,&ypos,&lenfie,\
	&fmaxk,&fmink,&lenfr);

/* read data from file */
specialfield = 0;
*data_points = 0;
while(1)	
	{
	a = fscanf(stockfile, "%s", temp);
	if(a == EOF)
		{
		fclose(stockfile);
		break;
		}

	value = (float)atof(temp);
	if(value < 0)
		{
		special_field_type = value;
		specialfield = 1;
		continue; /* get next field */
		}
	if(specialfield)
		{
		specialfield = 0;
		continue; /* scip */
		}
	if(value > 0) /* if valid */
		{
		closeprice = value;
		data[*data_points] = closeprice;
			
		special_field_type = 0;
		(*data_points)++;
		}
	}/* end while read values from file */

return(1);
}/* end function load_stock_prices */


int get_current_value(char *filename, float *current_value)
{
int a, b, i, j;
FILE *stockfile;
char fiwd[80], fimo[80], si[80], temp[80];
float trend, fmink, fmaxk;
int fida, af, pg, sp, xpos, ypos, lenfie, lenfr;
int special_field_type;
int special_field;
float  value, closeprice, openprice, minprice, maxprice, volume;
int header_flag;
char header[80];
int month, day, year;
char date[80];

if(debug_flag)
	{
	fprintf(stdout, "get_current_value(): arg filename=%s\n", filename); 
	}

/* argument check */
if(! filename) return 0;
if(! current_value) return 0;

stockfile = fopen(filename, "r");
if(! stockfile) 
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"get_interest(): cannot open file %s for read\n", filename);
		}
	fl_show_alert("Cannot open file", filename, "for read", 1);
	return (0);
	}

/* get header from file */
fscanf(stockfile,"%s %s %d %s %f %d %d %d %d %d %d %f %f %d",\
	fiwd,fimo,&fida,si,&trend,&af,&pg,&sp,&xpos,&ypos,&lenfie,\
	&fmaxk,&fmink,&lenfr);

special_field = 0;
while(1)	
	{
	a = fscanf(stockfile, "%s", temp);
	if(a == EOF)
		{
		fclose(stockfile);
		break;
		}
	value = (float)atof(temp);
	if(value < 0)
		{
		special_field_type = value;
		special_field = 1;
		continue; /* get next field */
		}
	if(special_field)
		{
		special_field = 0;
		continue; /* scip */
 		}
	if(value > 0) /* if valid */
		{
		closeprice = value;
		special_field_type = 0;
		}
	}/* end while read values from file */

/* the last one read is the current interest */
*current_value = closeprice;

return(1);
}/* end function get_current_value */


int month_to_digit(char *month)
{
int a;
int b;

/* month could take the form 01, or 1 */
b = 0;
if( isdigit(month[0]) ) b != 1;
if( isdigit(month[1]) ) b != 2;

/* test for 1 or 01 or 12 */
if( (b == 1) || (b == 3) )
	{
	a = atoi(month);
	if( (a > 0) && (a < 13) ) return a;
	return 0;
	}
/* a1 would be no month */
if(b == 2) return 0;

if(strncasecmp(month,"Jan", 3) == 0) return 1; 
if(strncasecmp(month,"Feb", 3) == 0) return 2;
if(strncasecmp(month,"Mar", 3) == 0) return 3; 
if(strncasecmp(month,"Maa", 3) == 0) return 3; 
if(strncasecmp(month,"Apr", 3) == 0) return 4; 
if(strncasecmp(month,"May", 3) == 0) return 5; 
if(strncasecmp(month,"Mei", 3) == 0) return 5; 
if(strncasecmp(month,"Jun", 3) == 0) return 6; 
if(strncasecmp(month,"Jul", 3) == 0) return 7; 
if(strncasecmp(month,"Aug", 3) == 0) return 8; 
if(strncasecmp(month,"Sep", 3) == 0) return 9; 
if(strncasecmp(month,"Oct", 3) == 0) return 10;
if(strncasecmp(month,"Okt", 3) == 0) return 10;
if(strncasecmp(month,"Nov", 3) == 0) return 11;
if(strncasecmp(month,"Dec", 3) == 0) return 12;

return 0;
}


int calculate_option_price(\
char *fundname,\
char *expiration_month,\
char *expiration_year,\
char *strike_price,\
float *days_to_expiration,\
float *current_riskfree_interest_rate,\
float *current_stock_price,\
float *daily_standard_deviation,\
double *yearly_standard_deviation,\
float *present_call_value,\
float *present_put_value\
)
{
int data_available_flag;
float option_strike_price;
double present_option_value;
float data[DATA_SIZE];
int data_points;
char temp[512];

/* argument check */
if(! fundname) return 0;
if(! expiration_month) return 0;
if(! expiration_year) return 0;
if(! strike_price) return 0;

if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): arg\n\
	fundname=%s expiration_month=%s expiration_year=%s\n",\
	fundname,  expiration_month, expiration_year);
	}

/*
calculate days to expiration, or get from input field if in manual mode.
*/
if(! option_calculator_form_days_to_expiration_manual_input_flag)
	{
	if(! get_days_to_expiration(\
	expiration_month, expiration_year, days_to_expiration) )/* already pointer */
		{
		fprintf(stdout,\
		"calculate_option_price(): cannot calculate days to expiration\n");
		return 0;
		}
	}
else /* manual */
	{
	*days_to_expiration =\
		 atof( (char *)fl_get_input(\
		 fdui -> option_calculator_form_days_to_expiration_input_field) );
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): *days_to_expiration=%.2f\n",\
	*days_to_expiration);
	}

/* get the option strike price */
option_strike_price = atof(strike_price);
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): option_strike_price=%.2f\n",\
	option_strike_price);
	}

/*
get current interest rate from file, or from input filed if in manual mode.
*/
if(! option_calculator_form_current_riskfree_interest_rate_manual_input_flag)
	{
	/* get the current interest rate */
	sprintf(temp, "%s%s.krs", file_path,\
		(char *)fl_get_input(\
		fdui -> option_calculator_form_riskfree_interest_file_input_field) );
 	/* arg 2 already a pointer */
	if(! get_current_value(temp, current_riskfree_interest_rate) ) return 0;
	/* convert from percentages to real values */
	*current_riskfree_interest_rate /= 100.0;
	}
else /* manual */
	{
	*current_riskfree_interest_rate =\
		atof( (char *)fl_get_input(\
		fdui -> option_calculator_form_current_riskfree_interest_rate_input_field) );
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): current risk free interest rate=%.6f\n",\
	(float *)current_riskfree_interest_rate);
	}

/*
in case close price manual, and daily or yearly deviation manual, no need
to load file.
*/
if( (! option_calculator_form_current_stock_price_manual_input_flag) &&\
   ( (! option_calculator_form_daily_standard_deviation_manual_input_flag ) ||\
    (!option_calculator_form_yearly_standard_deviation_manual_input_flag ) )
  )
	{
	/* get the historical close prices */
	sprintf(temp, "%s%s.krs", file_path, fundname);
	data_available_flag = 1;
	if(! load_stock_prices(temp, data, &data_points) )
		{
		data_available_flag = 0;
		}
	
	if(data_points < 2) data_available_flag = 0;

	/*
	if no data available (no .krs file?) go to manual mode for
	current stock price, daily std deviation, yearly std deviation
	*/
	if(! data_available_flag)
		{
		option_calculator_form_current_stock_price_manual_input_flag = 1;
		fl_set_button(fdui -> option_calculator_form_current_stock_price_button, 1);
		fl_set_object_label(\
		fdui -> option_calculator_form_current_stock_price_button, "MANUAL");
		option_calculator_form_daily_standard_deviation_manual_input_flag = 1;
		fl_set_button(fdui -> option_calculator_form_daily_standard_deviation_button, 1);
		fl_set_object_label(\
		fdui -> option_calculator_form_daily_standard_deviation_button, "MANUAL");
		option_calculator_form_yearly_standard_deviation_manual_input_flag = 1;
		fl_set_button(fdui -> option_calculator_form_yearly_standard_deviation_button, 1);
		fl_set_object_label(\
		fdui -> option_calculator_form_yearly_standard_deviation_button, "MANUAL");
		}
	/* show the number of data points loaded */
	sprintf(temp, "%d", data_points);
	fl_set_input(fdui -> option_calculator_form_data_points_input_field, temp);

	if(debug_flag)
		{
		fprintf(stdout,\
"calculate_option_price(): historical stock prices %d data points loaded\n",\
		data_points);
		}
	}

/*
calculate the daily deviation from historical prices, or get from input
field if in manual mode.
*/
if(! option_calculator_form_daily_standard_deviation_manual_input_flag)
	{
	/* calculate the daily standard deviation */
	*daily_standard_deviation = stddev(data, data_points);
	}
else /* manual */
	{
	*daily_standard_deviation =\
	 atof( (char *)fl_get_input(\
		fdui -> option_calculator_form_daily_standard_deviation_input_field) );
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): daily standard deviation=%f\n",\
	(float *)daily_standard_deviation);
	}

/*
calculate the yearly deviation from historical prices, or get from input
field if in manual mode.
*/
if(! option_calculator_form_yearly_standard_deviation_manual_input_flag)
	{
	/* calculate the yearly standard deviation */
	/* there are 252 trading days in a year */
	*yearly_standard_deviation = sqrt(252) * *daily_standard_deviation;
	}
else /* manual */
	{
	*yearly_standard_deviation =\
	 atof( (char *)fl_get_input(\
		fdui -> option_calculator_form_yearly_standard_deviation_input_field) );
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): yearly standard deviation=%f\n",\
	(float *)yearly_standard_deviation);
	}

/*
get the current stock price from last data point, or from input if in manual
mode.
*/
if(! option_calculator_form_current_stock_price_manual_input_flag)
	{
	/* get the current stock price */
	/* the last data point holds the most recent stock value */
	*current_stock_price = data[data_points - 1];	
	}
else
	{
	*current_stock_price =\
	atof( (char *)fl_get_input(\
	 fdui -> option_calculator_form_current_stock_price_input_field) );
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price(): current stock price=%.2f\n", current_stock_price);
	}

/* calculate the current option value */
if(! option_value_black_scholes\
 (
  *current_stock_price,\
  option_strike_price,\
  *current_riskfree_interest_rate,\
  *days_to_expiration,\
  *yearly_standard_deviation,\
  present_call_value, /* already a pointer */\
  present_put_value /* already a pointer */\
 ) )
	{
 	fprintf(stdout,\
 	"calculate_option_price(): option_value_black_scholes() failed\n");
	return 0; 	
 	}

if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price():\n\
	*days_to_expiration=%.2f\n\
	*current_riskfree_interest_rate=%f\n\
	*current_stock_price=%f\n\
	*daily_standard_deviation=%f\n\
	*yearly_standard_deviation=%f\n\
	present_call_value=%f\n\
	present_put_value=%f\n",\

	*days_to_expiration,\
	*current_riskfree_interest_rate,\
	*current_stock_price,\
	*daily_standard_deviation,\
	(float*)yearly_standard_deviation,\
	*present_call_value,\
	*present_put_value);
	}

return 1;
}/* end function calculate_option_price */


int new_option_calculation()
{
char *fundname;
char *call_put;
char *expiration_month;
char *expiration_year;
char *strike_price;
char temp[80];
float days_to_expiration;
float current_riskfree_interest_rate;
float current_stock_price;
float daily_standard_deviation;
double yearly_standard_deviation;

float present_call_value;
float present_put_value;
int have_all_data_flag;

/* clear output field (avoid confusion with previous data if error exit) */
if(! option_calculator_form_days_to_expiration_manual_input_flag)
	{
	fl_set_input(fdui -> option_calculator_form_days_to_expiration_input_field, "");
	}
if(!option_calculator_form_current_riskfree_interest_rate_manual_input_flag )
	{
	fl_set_input(fdui -> option_calculator_form_current_riskfree_interest_rate_input_field, "");
	}
else
	{
/*	fl_set_input(fdui -> option_calculator_form_riskfree_interest_file_input_field, "");*/
	}
if(! option_calculator_form_current_stock_price_manual_input_flag)
	{
	fl_set_input(fdui -> option_calculator_form_current_stock_price_input_field, "");
	}
if(! option_calculator_form_daily_standard_deviation_manual_input_flag)
	{
	fl_set_input(fdui -> option_calculator_form_daily_standard_deviation_input_field, "");
	}
fl_set_input(fdui -> option_calculator_form_data_points_input_field, "");
if(! option_calculator_form_yearly_standard_deviation_manual_input_flag)
	{
	fl_set_input(fdui -> option_calculator_form_yearly_standard_deviation_input_field, "");
	}
fl_set_input(fdui -> option_calculator_form_present_call_value_input_field, "");
fl_set_input(fdui -> option_calculator_form_present_put_value_input_field, "");

fundname = (char *)fl_get_input(fdui -> option_calculator_form_fundname_input_field);
expiration_month = (char *)fl_get_input(fdui -> option_calculator_form_month_input_field);
expiration_year = (char *)fl_get_input(fdui -> option_calculator_form_year_input_field);
strike_price = (char *)fl_get_input(fdui -> option_calculator_form_strike_price_input_field);

have_all_data_flag = 1;
/*if(strlen(fundname) < 2) have_all_data_flag = 0;*/
if(strlen(expiration_month) == 0) have_all_data_flag = 0;
if(strlen(expiration_year) < 4) have_all_data_flag = 0;
if(strlen(strike_price) == 0) have_all_data_flag = 0;
if(option_calculator_form_days_to_expiration_manual_input_flag)
	{
	if(strlen((char *)fl_get_input(\
	fdui -> option_calculator_form_days_to_expiration_input_field) ) == 0)
		{
		have_all_data_flag = 0;
		}
	}
if(option_calculator_form_current_riskfree_interest_rate_manual_input_flag)
	{
	if(strlen((char *)fl_get_input(\
	fdui -> option_calculator_form_current_riskfree_interest_rate_input_field) ) == 0)
		{
		have_all_data_flag = 0;
		}
	}
if(option_calculator_form_current_stock_price_manual_input_flag)
	{
	if(strlen((char *)fl_get_input(\
	fdui -> option_calculator_form_current_stock_price_input_field) ) == 0)
		{
		have_all_data_flag = 0;
		}
	}
if(option_calculator_form_daily_standard_deviation_manual_input_flag)
	{
	if(strlen((char *)fl_get_input(\
	fdui -> option_calculator_form_daily_standard_deviation_input_field) ) == 0)
		{
		if(! option_calculator_form_yearly_standard_deviation_manual_input_flag)
			{
			have_all_data_flag = 0;
			}
		}
	}
if(option_calculator_form_yearly_standard_deviation_manual_input_flag)
	{
	if(strlen((char *)fl_get_input(\
	fdui -> option_calculator_form_yearly_standard_deviation_input_field) ) == 0)
		{
		have_all_data_flag = 0;
		}
	}

if(! have_all_data_flag) return 0;

if(! calculate_option_price(\
 fundname,\
 expiration_month,\
 expiration_year,\
 strike_price,\
 &days_to_expiration,\
 &current_riskfree_interest_rate,\
 &current_stock_price,\
 &daily_standard_deviation,\
 &yearly_standard_deviation,\
 &present_call_value,\
 &present_put_value\
 ) ) return 0;

if(! option_calculator_form_days_to_expiration_manual_input_flag)
	{
	sprintf(temp, "%.2f", days_to_expiration);
	fl_set_input(fdui -> option_calculator_form_days_to_expiration_input_field, temp);
	}

if(! option_calculator_form_current_riskfree_interest_rate_manual_input_flag )
	{
	sprintf(temp, "%.6f", current_riskfree_interest_rate);
	fl_set_input(fdui -> option_calculator_form_current_riskfree_interest_rate_input_field,\
	temp);
	}

if(! option_calculator_form_current_stock_price_manual_input_flag)
	{
	sprintf(temp, "%.2f", current_stock_price);
	fl_set_input(fdui -> option_calculator_form_current_stock_price_input_field, temp);
	}

if(! option_calculator_form_daily_standard_deviation_manual_input_flag)
	{
	sprintf(temp, "%.6f", daily_standard_deviation);
	fl_set_input(fdui -> option_calculator_form_daily_standard_deviation_input_field, temp);
	}
	
if(! option_calculator_form_yearly_standard_deviation_manual_input_flag)
	{
	sprintf(temp, "%.6f", (float)yearly_standard_deviation);
	fl_set_input(fdui -> option_calculator_form_yearly_standard_deviation_input_field, temp);
	}
	
sprintf(temp, "%.2f", present_call_value);
fl_set_input(fdui -> option_calculator_form_present_call_value_input_field, temp);

sprintf(temp, "%.2f", present_put_value);
fl_set_input(fdui -> option_calculator_form_present_put_value_input_field, temp);

return 1;
}/* end function new_option_calculation */


int get_days_to_expiration2(\
int expiration_month, int expiration_year, float *days_to_expiration)
{
/*
This calculates the days until expiration
It uses the computer time as the current time.
The expiration is taken to happen on the third friday of the month.
Change as required.
*/

struct tm *petm;
time_t now, expiration_time;
int seconds_per_day;
int mday;
int friday_cnt;

if(debug_flag)
	{
	fprintf(stdout,\
"get_days_to_expiration2(): arg expiration_month=%d expiration_year=%d\n",\
	expiration_month, expiration_year);
	}

/* argument check */
if(expiration_month < 1) return 0;
if(expiration_month > 12) return 0;
if(expiration_year < 1900) return 0;

/* get computer time */
now = time(NULL);

/* allocate space for a structure tm */
petm = (struct tm *) malloc(sizeof(struct tm) );
if(! petm)
	{
	if(debug_flag)
		{
		fprintf(stdout,\
	"get_days_to_expiration2(): cannot allocate space for structure petm\n");
		}
	return 0;
	}

/*
need tm_wday for check on friday (sunday = 0), then iterate forward :)
so friday = 5
*/
friday_cnt = 0;
mday = 1;

while(1)
	{
	if(debug_flag)
		{
		fprintf(stdout, "get_days_to_expiration2(): mday=%d\n", mday);
		}

	petm -> tm_sec = 0;/* unknown */
	petm -> tm_min = 0;/* unknown */
	petm -> tm_hour = 12;/* mid day */
	petm -> tm_mday = mday; /* 1 to 31, use 3rd friday in month */
	petm -> tm_mon = expiration_month - 1;
	petm -> tm_year = expiration_year - 1900;
	petm -> tm_isdst = -1;/* daylight savings time n.a. */
	/* the following are not implemented (see /usr/include/time.h) */
	/* petm -> tm_gmtoff = 0;*/
	/* strcp(petm -> tm_zone, "");/* time zone 3 letters */

	expiration_time = mktime( (struct tm *)petm);

	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration2(): expiration_time=%lu\n", expiration_time);
		}

	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration2(): at wday=%d\n", petm -> tm_wday);
		}

	/* check for friday  (day 5 in week) */
	
	if(debug_flag)
		{
		fprintf(stdout,\
		"get_days_to_expiration2(): petm -> tm_wday=%d\n",\
		petm -> tm_wday);
		}

	if(petm -> tm_wday == 5)
		{
		if(debug_flag)
			{
			fprintf(stdout,\
			"get_days_to_expiration2(): iterating friday_cnt=%d\n",\
			friday_cnt);
			}
  		friday_cnt++;
		/* third friday in expiration month is expiration day */
		if(friday_cnt == 3) break;
		}
	mday++;
	if(mday > 31)
		{
		free(petm);
		return 0;
		}
	}/* end while looking for 3rd friday in expiration month */

/* calculate days in year to third friday in expiration month */
seconds_per_day = 60 * 60 * 24;
*days_to_expiration = (float) (expiration_time - now) / seconds_per_day;
if(debug_flag)
	{
	fprintf(stdout,\
	"get_days_to_expiration2(): days_to_expiration=%.2f\n",\
	*days_to_expiration);
	}

free(petm);

return 1;
} /* end function get_days_to_expiration2 */


int calculate_option_price_fast(\
char *fundname,\
char *expiration_month,\
char *expiration_year,\
char *strike_price,\
float *days_to_expiration,\
float *current_riskfree_interest_rate,\
float *current_stock_price,\
float *daily_standard_deviation,\
double *yearly_standard_deviation,\
float *present_call_value,\
float *present_put_value\
)
{
float option_strike_price;
double present_option_value;
char temp[512];
int get_new_data;

/* argument check */
if(! fundname) return 0;
if(! expiration_month) return 0;
if(! expiration_year) return 0;
if(! strike_price) return 0;

if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price_fast(): arg\n\
	fundname=%s expiration_month=%s expiration_year=%s\n",\
	fundname,  expiration_month, expiration_year);
	}

/* calculate days to expiration */
if(! get_days_to_expiration(\
expiration_month, expiration_year, days_to_expiration) )/* already pointer */
	{
	if(debug_flag)
		{
		fprintf(stdout,\
		"calculate_option_price_fast():cannot calculate days to expiration\n\
		fundname=%s expiration_month=%s expiration_year=%s",\
		fundname, expiration_month, expiration_year);
		}
	
	return 0;
	}
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price_fast(): *days_to_expiration=%.2f\n",\
	*days_to_expiration);
	}

/* get the option strike price */
option_strike_price = atof(strike_price);
if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price_fast(): option_strike_price=%.2f\n",\
	option_strike_price);
	}

/* only load new data if new fundname */
/* this avoids old_fundname zero ptr in strcmp resulting in segfault */
get_new_data = 0;
if(! old_fundname) get_new_data = 1;
else if(strcmp(fundname, old_fundname) != 0) get_new_data = 1;
if(get_new_data)
	{
	/* get the historical close prices */
	sprintf(temp, "%s%s.krs", "/root/stockvac/", fundname);
	if(! load_stock_prices(temp, data, &data_points) ) return 0;
	if(debug_flag)
		{
		fprintf(stdout,\
"calculate_option_price_fast(): historical stock prices %d data points loaded\n",\
		data_points);
		}
	
	if(old_fundname) free(old_fundname);
	old_fundname = strsave(fundname);
	if(! old_fundname) return 0;
	
	if(data_points < 2) return 0;

	/* calculate the daily standard deviation */
	*daily_standard_deviation = stddev(data, data_points);
	if(debug_flag)
		{
		fprintf(stdout,\
		"calculate_option_price_fast(): daily standard deviation=%f\n",\
		*daily_standard_deviation);
		}

	/* calculate the yearly standard deviation */
	/* there are 252 trading days in a year */
	*yearly_standard_deviation = sqrt(252) * *daily_standard_deviation;
	if(debug_flag)
		{
		fprintf(stdout,\
		"calculate_option_price_fast(): yearly standard deviation=%f\n",\
		(float)*yearly_standard_deviation);
		}

	/* get the current stock price */
	/* the last data point holds the most recent stock value */
	*current_stock_price = data[data_points - 1];	
	if(debug_flag)
		{
		fprintf(stdout,\
		"calculate_option_price_fast(): current stock price=%.2f\n",\
		*current_stock_price);
		}

	/* save for later */
	old_yearly_standard_deviation = *yearly_standard_deviation;
	old_current_stock_price = *current_stock_price;
	}/* end if fundname != old_fundname */
else
	{
	*yearly_standard_deviation = old_yearly_standard_deviation;
	*current_stock_price = old_current_stock_price;
	}


/* calculate the current option value */
if(! option_value_black_scholes\
 (
  *current_stock_price,\
  option_strike_price,\
  *current_riskfree_interest_rate,\
  *days_to_expiration,\
  *yearly_standard_deviation,\
  present_call_value, /* already a pointer */\
  present_put_value /* already a pointer */\
 ) )
	{
 	fprintf(stdout,\
 	"calculate_option_price_fast(): option_value_black_scholes() failed\n");
	return 0; 	
 	}

if(debug_flag)
	{
	fprintf(stdout,\
	"calculate_option_price_fast():\n\
	*days_to_expiration=%.2f\n\
	*current_riskfree_interest_rate=%f\n\
	*current_stock_price=%f\n\
	*daily_standard_deviation=%f\n\
	*yearly_standard_deviation=%f\n\
	present_call_value=%f\n\
	present_put_value=%f\n",\

	*days_to_expiration,\
	*current_riskfree_interest_rate,\
	*current_stock_price,\
	*daily_standard_deviation,\
	(float)*yearly_standard_deviation,\
	*present_call_value,\
	*present_put_value);
	}

return 1;
}/* end function calculate_option_price_fast */


